#include "PmergeMe.hpp"
#include <algorithm>
#include <cctype>
#include <cstddef>
#include <cstdlib>
#include <string>
#include <time.h> 
#include <deque>
#include <iostream>
#include <vector>

template<typename T>
void display(const T& array)
{
	for (size_t i = 0; i != array.size(); i++)
		if (array[i] != -1)
			std::cout << array[i] << " ";
}

template <typename T>
int parsing_arg(const char* str, T& array)
{
	int number = -1;
	for (size_t i = 0; true; i++)
	{
		if (std::isdigit(str[i]))
		{
			if (number == -1)
				number = 0;
			number = number * 10 + str[i] - '0';
		}
		else if (str[i] == '\0' || str[i] == ' ')
		{
			if (number != -1)
			{
				array.push_back(number);
				number = -1;
			}
			if (str[i] == '\0')
				return 0;
		}
		else
		{
			std::cout << "Error: " << str[i] << i << " invalid character." << std::endl;
			return 1;
		}
	}
}

template <typename T>
int parsing_args(const char* const * args, size_t len, T& array)
{
	for (size_t i = 0; i != len; i++)
	{
		if (parsing_arg(args[i], array))
			return 1;
	}
	return 0;
}

template <typename T>
int parsing_duplicate(T& array)
{
	for (size_t i = 0; i + 1 != array.size(); i++)
	{
		for (size_t j = i + 1; j != array.size(); j++)
		{
			if (array[i] == array[j])
			{
				std::cout << "Error: " << array[i] << " is duplicated." << std::endl;
				return 1;
			}
		}
	}
	return 0;
}

template <typename T>
int parsing(const char* const* args, size_t len, T& array)
{
	return parsing_args(args, len, array) || parsing_duplicate(array);
}

int main(int ac, char** av)
{
	{
		std::vector<int> array;

		if (parsing(av + 1, (size_t) ac - 1, array))
			return 1;

		std::cout << "Before: ";
		display(array);
		std::cout << std::endl;
		
		const clock_t time_vec_start = clock();
		
		PmergeMe(array);
		
		const clock_t time_vec_stop = clock();
		
		std::cout << "After: ";
		display(array);
		std::cout << std::endl;
		std::cout << "Time to process a range of " << array.size() <<" elements with std::vector : " << ((double) (time_vec_stop - time_vec_start) / CLOCKS_PER_SEC) * 1000000 << " us" << std::endl;
	}
	{
		std::deque<int> array;

		if (parsing(av + 1, (size_t) ac - 1, array))
			return 1;

		const clock_t time_vec_start = clock();
		
		PmergeMe(array);
		
		const clock_t time_vec_stop = clock();
		
		std::cout << "Time to process a range of " << array.size() <<" elements with std::deque : " << ((double) (time_vec_stop - time_vec_start) / CLOCKS_PER_SEC) * 1000000 << " us" << std::endl;
	}
}
